(* Mathematica Package *)

(* Created by the Wolfram Workbench Jul 5, 2006 *)

BeginPackage["SteinerTripleSystems`"]
(* Exported symbols added here with SymbolName::usage *) 
GetAllPaths::usage = "GetAllPaths[s] returns a set will all paths of triples with s being a set triples."
GetPaths::usage = "GetPaths[ x, p, s] sows paths continuing path p and element x in set s of triples."
SelectTriples::usage = "SelectTriples[x, l] returns the subset of triples in l that contain the element x."
SteinerTripleSystemQ::usage = "SteinerTripleSystemQ[s] Returns true if the set of triples s is a Steiner Triple System, false otherwise."
NumberOfTriples::usage = "NumberOfTriples[o] returns the number of triples that a Steiner Triple System of order o contains."
SteinerTripleSystemOrderQ::usage = "SteinerTripleSystemOrderQ[o] returns true if the order o is a valid order for a Steiner Triple System, false otherwise."
SystemSignatureMatrix::usage = "SystemSignatureMatrix[s] returns the System Signature Signature Matrix of the Steiner Triple System s."
CompositeSystemSignatureMatrix::usage = "CompositeSystemSignatureMatrix[s_]"
SystemSignature::usage = "SystemSignature[s] returns the diagonal of the System Signature Matrix of the Steiner Triple System s."
SortSignatureMatrix::usage = "SortSignatureMatrix[s] returns the System Signature Matrix sorted."
PentagonalWaveSequence::usage = "PentagonalWaveSequence[n] returns the nth integer in the Pentagonal Wave Sequence of the First Kind."
SelectionMatrix::usage = "SelectionMatrix[m] returns the Selection Matrix derived from the System Signature Matrix s."
SteinerTripleSystem::usage = "SteinerTripleSystem[sM] returns a Steiner Triple System obtained from the selection matrix sM."
OldSteinerTripleSystem::usage = "OldSteinerTripleSystem[sM] returns a Steiner Triple System obtained from the selection matrix sM."
OldAllSteinerTripleSystems::usage = "OldAllSteinerTripleSystems[sM] Return all Steiner Triple Systems obtained from the selection matrix sM."

MostCommonElement::usage = "MostCommonElement returns the most common element in the set of triples."

Begin["`Private`"]
(* Implementation of the package *)
OldSelectTriples[x_, l_] :=
    Block[ {
       partial1 = Union[Table[InSet[x, l[[i]]], {i, 1, Length[l]}]]},
        If[ MemberQ[partial1, Null],
            Rest[partial1],
            partial1
        ]
    ];

SelectTriples[x_, l_] := 
    Block[ {
       resultSet = {}},
        Map[If[MemberQ[#, x], AppendTo[resultSet, #]] &, l];
        resultSet
    ]

GetPaths[x_, p_, s_] :=
    Block[ {
       chooseFrom = SelectTriples[x,s],
       i = 0,
       j = 0},
        If[ chooseFrom != {},
            For[i = 1, i <= Length[chooseFrom], i++,
                Block[ {perms = Permutations[Complement[chooseFrom[[i]], {x}]]},
                    For[j = 1, j <= Length[perms], j++,
                          Block[ {ct = Prepend[perms[[j]], x]},
                              GetPaths[ct[[3]], Append[p, ct], Complement[s, SelectTriples[ct[[2]], s], SelectTriples[x, s]]]
                          ]]
                ]],
            Sow[p]
        ]
    ];
              
GetAllPaths[s_] :=
    Block[ {
       elements = Union[Flatten[s]]},
        Reap[For[m = 1, m <= Length[elements], m++, GetPaths[elements[[m]], {}, s]]][[2, 1]]
    ];
        
SteinerTripleSystemQ[s_] :=
    Subsets[Union[Flatten[s]], {2}] == Sort[Flatten[Map[Map[Sort, Subsets[#, {2}]] & , s], 1]];
  
NumberOfTriples[o_] :=
    If[ SteinerTripleSystemOrderQ[o],
        Binomial[o, 2]/3,
        0
    ]; 

SteinerTripleSystemOrderQ[o_] :=
    Mod[o, 6] == 1 || Mod[o, 6] == 3;

MostCommonElement[s_] :=
    Reverse[Sort[Map[{Length[#], #[[1]]} & , Split[Sort[Flatten[s]]]]]][[1, 2]];
        
SystemSignatureMatrix[s_] :=
    Block[ {
       theSet = s,
       inputSet = Union[Flatten[s]], 
       theRules,
       colorSets = {}, 
       counter = 0, 
       maxLength = Length[s], 
       construx = {},
       mapping = {}},
        While[Length[theSet] > 0 && counter <= maxLength,
            If[ Length[Sort[Flatten[SelectTriples[MostCommonElement[theSet], theSet]]]] > 0,
                AppendTo[colorSets, Sort[Flatten[SelectTriples[MostCommonElement[theSet], theSet]]]]
            ];
            theSet = Complement[theSet, SelectTriples[MostCommonElement[theSet], theSet]];
            counter++];
        theRules = Table[inputSet[[i]] -> i - 1, {i, 11, Length[inputSet]}];
        mapping = colorSets /. theRules;
        construx = Table[0, {j, 1, Length[inputSet]}, {i, 1, Length[mapping]}];
        Do[construx[[mapping[[i]][[j]] + 1, i]]++, {i, 1, Length[mapping]}, {j, 1, Length[mapping[[i]]]}];
        construx
    ]

CompositeSystemSignatureMatrix[s_] :=
    Block[ {
       theSet = s,
       inputSet = Union[Flatten[s]], 
       theRules,
       colorSets = {}, 
       counter = 1, 
       maxLength = Length[s], 
       construx = {},
       inputSweeper = 1,
       mapping = {}},
        While[Length[theSet] > 0 && counter <= maxLength,
            If[ Length[Sort[Flatten[SelectTriples[inputSet[[inputSweeper]], theSet]]]] > 0,
                colorSets = Append[colorSets,Sort[Flatten[SelectTriples[inputSet[[inputSweeper]], theSet]]]]
            ];
            theSet = Complement[theSet, SelectTriples[inputSet[[inputSweeper++]], theSet]];
            counter++];
        theRules = Table[inputSet[[i]] -> i - 1, {i, 11, Length[inputSet]}];
        mapping = colorSets /. theRules;
        construx = Table[0, {j, 1, Length[inputSet]}, {i, 1, Length[mapping]}];
        Do[construx[[mapping[[i]][[j]] + 1, i]]++, {i, 1, Length[mapping]}, {j, 1, Length[mapping[[i]]]}];
        construx
    ]
        
SystemSignature[s_] :=
    MapThread[Max, SystemSignatureMatrix[s]];

SortSignatureMatrix[s_] :=
    Map[#[[2]] &, Reverse[Sort[Map[{If[ #[[-1]] != 0,
                                        #[[-1]],
                                        Map[Length, Split[#]][[-1]] + #[[-Map[Length, Split[#]][[-1]] - 1]] 10
                                    ], #} &, s]]]];
    
PentagonalWaveSequence[n_] :=
    If[ OddQ[n],
        n(3n - 1)/2,
        n(3n + 1)/2
    ];

ColumnInfo[column_] :=
    Block[ {tempCol = Position[column, _?(0 < # &)] - 1},
        {Flatten[tempCol[[1]]][[1]], Flatten[Rest[tempCol]]}
    ];

SelectionMatrix[m_] :=
    Map[ColumnInfo[Reverse[Transpose[m]][[#]]] &, Table[n, {n, 1, Length[Reverse[Transpose[m]]]}]];

GetSelectedPairs[s_, nPairs_] :=
    Block[ {
       tempSet = Union[Map[If[ Length[Union[Flatten[#]]] == 2 nPairs,
                               #
                           ] &, Subsets[s, {nPairs}]]]},
        If[ MemberQ[tempSet, Null],
            Rest[tempSet],
            tempSet
        ]
    ];
            
RandomRelist[x_List] :=
    Block[ {
       n = x, 
       p},
        Do[
            p = Random[Integer, {1, i}];
            n[[{p, i}]] = n[[{i, p}]],
            {i, Length[x]}];
        n
    ];

GetPairs[wPSet_, p_, l_] :=
    Module[ {(*tempPairs = Subsets[l, {2}][[Range[Length[l] - 1]]],*)
    tempPairs = Map[{First[l], #}&, Complement[l, {First[l]}]],
    validatedPairs={}},
    	Map[If[MemberQ[wPSet, #], AppendTo[validatedPairs, #]] &, tempPairs];
    	If[Length[validatedPairs] == 0, Return[p]];
        If[ Length[l] == 2,
            Return[{Join[p, {l}]}],
            Return[Flatten[Map[GetPairs[wPSet, Join[p, {#}], Complement[l, #]] &, validatedPairs], 1]]
        ]
    ]

ExtractProperPairSets[pSet_, eSet_] :=
    Module[ {
        tempPSets = GetPairs[pSet, {}, eSet],
        validPSets = {}},
        Map[If[ Apply[And, Map[MemberQ[pSet,#] &,#]],
                AppendTo[validPSets, #]
            ] &, tempPSets];
        validPSets
    ]

SteinerTripleSystem[sM_List, cIndex_, pSet_, tSet_] :=
    Block[ {
       tempPairsSet, 
       allPossiblePairs,
       randomChoices, 
       returnedValue = Null,
       selector = 1,
       pairsSet = pSet,
       tripleSet = tSet,
       selectedPairs},
        (*Print["Index -> "<>ToString[cIndex]<>" with "<>ToString[Length[pSet]]<>" pairs."];*)
        (*Print[pSet];*)
        allPossiblePairs = ExtractProperPairSets[pSet, sM[[cIndex]][[2]]];
        (*allPossiblePairs = GetPairs[pSet, {}, sM[[cIndex]][[2]]];*)
        (*Print["  Possible pairs "<>ToString[Length[allPossiblePairs]]];*)
        If[ Length[allPossiblePairs] == 0,
            Print["    No Possible Pairs at " , cIndex]
        ];
        If[ Length[allPossiblePairs] == 0,
            Return[{}]
        ];
        randomChoices = RandomRelist[Table[i, {i, Length[allPossiblePairs]}]];
        For[selector = 1, selector <= Length[allPossiblePairs], selector++,
            (*Print["-----> "<>ToString[selector]];*)
            selectedPairs = allPossiblePairs[[randomChoices[[selector]]]];
            tripleSet = Map[Append[#, sM[[cIndex]][[1]]]&, selectedPairs];
            If[ cIndex >= Length[sM],
                Return[Join[tSet, tripleSet]]
            ];
            returnedValue = SteinerTripleSystem[sM, cIndex + 1, Complement[pairsSet, Sort[Flatten[Map[Map[Sort, Subsets[#, {2}]] & , tripleSet], 1]]], Join[tSet, tripleSet]];
            If[ Length[returnedValue] != 0,
                Return[returnedValue]
            ]];
        Return[{}]
    ];

SteinerTripleSystem[sM_List] :=
    SteinerTripleSystem[sM, 1, Subsets[Union[Flatten[sM]], {2}], {}];
    
OldSteinerTripleSystem[sM_List, cIndex_, pSet_, tSet_] :=
    Block[ {
       tempPairsSet, 
       allPossiblePairs,
       randomChoices, 
       returnedValue = Null,
       selector = 1,
       pairsSet = pSet,
       tripleSet = tSet,
       numberOfPairs = (Length[sM[[cIndex]][[2]]])/2,
       tempPairs = Intersection[Subsets[sM[[cIndex]][[2]], {2}], pSet],
       selectedPairs},
        If[ numberOfPairs == 0,
            Return[{}]
        ];
        Print[cIndex];
        allPossiblePairs = GetSelectedPairs[tempPairs, numberOfPairs];
        If[ Length[allPossiblePairs] == 0,
            Print["No Possible Pairs at " , cIndex]
        ];
        If[ Length[allPossiblePairs] == 0,
            Return[{}]
        ];
        randomChoices = RandomRelist[Table[i, {i, Length[allPossiblePairs]}]];
        For[selector = 1, selector <= Length[allPossiblePairs], selector++,
            selectedPairs = allPossiblePairs[[randomChoices[[selector]]]];
            tripleSet = Map[Append[#, sM[[cIndex]][[1]]]&, selectedPairs];
            (*Print["Index=", cIndex," Selector=", selector,  " total pairs = ", Length[allPossiblePairs], " tripleSet=", tripleSet];*)
            tempPairsSet = Complement[pairsSet, Sort[Flatten[Map[Map[Sort, Subsets[#, {2}]] & , tripleSet], 1]]];
            If[ cIndex >= Length[sM],
                Return[tripleSet]
            ];
            returnedValue = OldSteinerTripleSystem[sM, cIndex + 1, tempPairsSet, tripleSet];
            (*Print["Right after the call returnedValue = ",returnedValue, "The Length is ", Length[allPossiblePairs]];*)
            (*If[returnedValue != Null,
                tripleSet = Join[tripleSet,
                returnedValue]; 
              Break[ ]];*)
            If[ Length[returnedValue] != 0,
                Return[Join[tripleSet, returnedValue]]
            ];];
        (*Print["After For Loop Returned Value = ", returnedValue, " TripleSet = ", tripleSet, " Index=", cIndex, " Selector=", selector, " Length=",Length[allPossiblePairs]];\)*)
        Return[{}]
    ];

OldSteinerTripleSystem[sM_List] :=
    OldSteinerTripleSystem[sM, 1, Subsets[Union[Flatten[sM]], {2}], {}];

OldAllSteinerTripleSystems[sM_List, cIndex_, pSet_, tSet_] :=
    Block[ {
       tempPairsSet, 
       allPossiblePairs,
       randomChoices, 
       returnedValue = Null,
       selector = 1,
       tripleSet,
       numberOfPairs = (Length[sM[[cIndex]][[2]]])/2,
       tempPairs = Intersection[Subsets[sM[[cIndex]][[2]], {2}], pSet],
       selectedPairs},
        If[ numberOfPairs == 0,
            Return[{}]
        ];
        (*Print[cIndex];*)
        allPossiblePairs = GetSelectedPairs[tempPairs, numberOfPairs];
        (*If[Length[allPossiblePairs] == 0, 
            Print["No Possible Pairs at " , cIndex]];*)
        If[ Length[allPossiblePairs] == 0,
            Return[{}]
        ];
        For[selector = 1, selector <= Length[allPossiblePairs], selector++,
            selectedPairs = allPossiblePairs[[selector]];
            tripleSet = Map[Append[#, sM[[cIndex]][[1]]]&, selectedPairs];
            tempPairsSet = Complement[pSet, Sort[Flatten[Map[Map[Sort, Subsets[#, {2}]] & , tripleSet], 1]]];
            If[ Length[tSet] > 0,
                tripleSet = Join[tSet, tripleSet]
            ];
            If[ cIndex >= Length[sM],
                Sow[tripleSet];
                Return[{}]
            ];
            OldAllSteinerTripleSystems[sM, cIndex + 1, tempPairsSet, tripleSet];
            ];
        Return[{}]
    ];
        
OldAllSteinerTripleSystems[sM_List] :=
    Reap[OldAllSteinerTripleSystems[sM, 1, Subsets[Union[Flatten[sM]], {2}], {}]][[2,1]];

End[]

EndPackage[]

